Index: apps/app_macro.c
===================================================================
RCS file: /usr/cvsroot/asterisk/apps/app_macro.c,v
retrieving revision 1.16
diff -u -r1.16 app_macro.c
--- apps/app_macro.c	2 Oct 2004 00:58:31 -0000	1.16
+++ apps/app_macro.c	23 Nov 2004 00:13:38 -0000
@@ -18,6 +18,7 @@
 #include <asterisk/pbx.h>
 #include <asterisk/module.h>
 #include <asterisk/options.h>
+#include <asterisk/config.h>
 #include <asterisk/utils.h>
 #include <asterisk/lock.h>
 #include <stdlib.h>
@@ -42,17 +43,26 @@
 "termination, Macro will attempt to continue at priority\n"
 "MACRO_OFFSET + N + 1 if such a step exists, and N + 1 otherwise.\n";
 
+static char *if_descrip =
+"  MacroIf(<expr>?label_a[|arg1][:label_b[|arg1]]):\n"
+"Executes macro defined in <label_a> if <expr> is true\n"
+"(otherwise <label_b> if provided)\n";
+
 static char *app = "Macro";
+static char *if_app = "MacroIf";
 
 static char *synopsis = "Macro Implementation";
+static char *if_synopsis = "Conditional Macro Implementation";
 
 STANDARD_LOCAL_USER;
 
 LOCAL_USER_DECL;
 
+#define ALL_DONE(u,ret) {LOCAL_USER_REMOVE(u); return ret;}
+
 static int macro_exec(struct ast_channel *chan, void *data)
 {
-  char tmp[256] = "";
+  char *tmp;
   char *cur, *rest;
   char *macro;
   char fullmacro[80];
@@ -72,17 +82,20 @@
   char *save_macro_context;
   char *save_macro_priority;
   char *save_macro_offset;
+  struct localuser *u;
   
   if (!data || ast_strlen_zero(data)) {
     ast_log(LOG_WARNING, "Invalid Macro incantation\n");
     return 0;
   }
-  strncpy(tmp, data, sizeof(tmp) - 1);
+  LOCAL_USER_ADD(u);
+
+  tmp = ast_strdupa((char *) data);
   rest = tmp;
   macro = strsep(&rest, "|");
   if (!macro || ast_strlen_zero(macro)) {
   	ast_log(LOG_WARNING, "Invalid macro name specified\n");
-	return 0;
+	ALL_DONE(u,0);
   }
   snprintf(fullmacro, sizeof(fullmacro), "macro-%s", macro);
   if (!ast_exists_extension(chan, fullmacro, "s", 1, chan->cid.cid_num)) {
@@ -90,7 +103,7 @@
 		ast_log(LOG_WARNING, "No such context '%s' for macro '%s'\n", fullmacro, macro);
 	else
 	  	ast_log(LOG_WARNING, "Context '%s' for macro '%s' lacks 's' extension, priority 1\n", fullmacro, macro);
-	return 0;
+	ALL_DONE(u,0);
   }
   /* Save old info */
   oldpriority = chan->priority;
@@ -220,17 +233,46 @@
 
   pbx_builtin_setvar_helper(chan, "MACRO_OFFSET", save_macro_offset);
   if (save_macro_offset) free(save_macro_offset);
-  return res;
+  ALL_DONE(u, res);
 }
 
+static int macroif_exec(struct ast_channel *chan, void *data) {
+	char *expr = NULL, *label_a = NULL, *label_b = NULL;
+	struct localuser *u;
+	int res = 0;
+
+	LOCAL_USER_ADD(u);
+	if((expr = ast_strdupa((char *) data))) {
+		if ((label_a = strchr(expr, '?'))) {
+			*label_a = '\0';
+			label_a++;
+			if ((label_b = strchr(label_a, ':'))) {
+				*label_b = '\0';
+				label_b++;
+			}
+			if (ast_true(expr))
+				macro_exec(chan, label_a);
+			else if (label_b) 
+				macro_exec(chan, label_b);
+			
+		} else
+			ast_log(LOG_WARNING, "Invalid Syntax.\n");
+	} else 
+		ast_log(LOG_ERROR, "Out of Memory!\n");
+
+	ALL_DONE(u, res);
+}
+			
 int unload_module(void)
 {
 	STANDARD_HANGUP_LOCALUSERS;
+	ast_unregister_application(if_app);
 	return ast_unregister_application(app);
 }
 
 int load_module(void)
 {
+	ast_register_application(if_app, macroif_exec, if_synopsis, if_descrip);
 	return ast_register_application(app, macro_exec, synopsis, descrip);
 }
 
